// PrefsDialog.cpp : implementation file
//

#include "stdafx.h"
#include "prefs.h"
#include "PrefsDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPrefsDialog dialog


CPrefsDialog::CPrefsDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CPrefsDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPrefsDialog)
	//}}AFX_DATA_INIT

   m_iCurPage = -1;
   m_pages.RemoveAll();
}

/////////////////////////////////////////////////////////////////////////////

CPrefsDialog::~CPrefsDialog()
{
	// clean up
   for (int i=0;i<m_pages.GetSize();i++)
   {
      pageStruct *pPS = (pageStruct *)m_pages.GetAt(i);
      if (pPS)
         delete pPS;
   }
}

/////////////////////////////////////////////////////////////////////////////

void CPrefsDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPrefsDialog)
	DDX_Control(pDX, IDC_PAGE_TREE, m_pageTree);
	DDX_Control(pDX, IDC_DLG_FRAME, m_boundingFrame);
	DDX_Control(pDX, IDC_CAPTION_BAR, m_captionBar);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPrefsDialog, CDialog)
	//{{AFX_MSG_MAP(CPrefsDialog)
	ON_WM_CREATE()
	ON_NOTIFY(TVN_SELCHANGED, IDC_PAGE_TREE, OnSelchangedPageTree)
	ON_NOTIFY(TVN_GETDISPINFO, IDC_PAGE_TREE, OnGetdispinfoPageTree)
	ON_BN_CLICKED(IDC_PHELP, OnPhelp)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_CHANGE_PAGE, OnChangePage)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPrefsDialog message handlers

/////////////////////////////////////////////////////////////////////////////

BOOL CPrefsDialog::PreTranslateMessage(MSG* pMsg) 
{
	ASSERT(pMsg != NULL);
   ASSERT_VALID(this);
   ASSERT(m_hWnd != NULL);

	if (CWnd::PreTranslateMessage(pMsg))
      return TRUE;

   // don't translate dialog messages when 
   // application is in help mode
   CFrameWnd* pFrameWnd = GetTopLevelFrame();
   if (pFrameWnd != NULL && pFrameWnd->m_bHelpMode)
      return FALSE;

   // ensure the dialog messages will not
   // eat frame accelerators
   pFrameWnd = GetParentFrame();
   while (pFrameWnd != NULL)
   {
      if (pFrameWnd->PreTranslateMessage(pMsg))
         return TRUE;
      pFrameWnd = pFrameWnd->GetParentFrame();
   }

   return PreTranslateInput(pMsg);

}

/////////////////////////////////////////////////////////////////////////////

int CPrefsDialog::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CDialog::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// create all of our pages
	for (int i=0;i<m_pages.GetSize();i++)
   {
      pageStruct *pPS = (pageStruct *)m_pages.GetAt(i);
      ASSERT(pPS);
      pPS->pDlg->Create(pPS->id, this);
   }
	
	return 0;
}

/////////////////////////////////////////////////////////////////////////////

BOOL CPrefsDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();

   // where will the dlgs live?
   m_boundingFrame.GetWindowRect(m_frameRect);
   ScreenToClient(m_frameRect);
   m_frameRect.DeflateRect(2,2);

	// set some styles for the pretty page indicator bar
   m_captionBar.m_textClr     = ::GetSysColor(COLOR_3DFACE);
	m_captionBar.m_fontWeight  = FW_BOLD;
	m_captionBar.m_fontSize    = 14;
   m_captionBar.m_csFontName  = "Verdana";
	m_captionBar.SetConstantText("ThumbNailer");

	// fill the tree
   for (int i=0;i<m_pages.GetSize();i++)
   {
      pageStruct *pPS = (pageStruct *)m_pages.GetAt(i);
      if (pPS)
      {
         TV_INSERTSTRUCT tvi;

         // find this node...
         tvi.hParent = FindHTREEItemForDlg(pPS->pDlgParent);

         tvi.hInsertAfter = TVI_LAST;
         tvi.item.cchTextMax = 0;
         tvi.item.pszText = LPSTR_TEXTCALLBACK;
         tvi.item.lParam = (long)pPS;
         tvi.item.mask = TVIF_PARAM | TVIF_TEXT;

         HTREEITEM hTree = m_pageTree.InsertItem(&tvi);
      }
   }

	// start with page 0
   if (ShowPage(0))
   {
      m_iCurPage = 0;   
   }

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

/////////////////////////////////////////////////////////////////////////////

HTREEITEM CPrefsDialog::FindHTREEItemForDlg(CDialog *pParent)
{
	if (pParent==NULL)
	{
		return TVI_ROOT;
	}

   HTREEITEM hItem = m_pageTree.GetRootItem();
   HTREEITEM htiCur = m_pageTree.GetNextItem( hItem );

   if( htiCur == NULL )
   {
      htiCur = m_pageTree.GetRootItem();
   }

   while( htiCur )
   {
      pageStruct *pPS  = (pageStruct *)m_pageTree.GetItemData( htiCur );
      if (pPS)
      {
         if (pPS->pDlg == pParent)
            return htiCur;
      }
      
      htiCur = m_pageTree.GetNextItem( htiCur );

      if( htiCur == NULL )
      {
         htiCur = m_pageTree.GetRootItem();
      }
   }

   return TVI_ROOT;
}

/////////////////////////////////////////////////////////////////////////////

LONG CPrefsDialog::OnChangePage(UINT u, LONG l)
{
   if (ShowPage(u))
   {
      m_iCurPage = u;   
   }

	return 0L;
}

/////////////////////////////////////////////////////////////////////////////

bool CPrefsDialog::AddPage(CDialog &dlg, UINT id, const char *pCaption, CDialog * pDlgParent /*=NULL*/)
{
	if (m_hWnd)
	{
		// can't add once the window has been created
		ASSERT(0);
		return false;
	}

   pageStruct *pPS = new pageStruct;
   pPS->pDlg = &dlg;
   pPS->id = id;
   pPS->csCaption = pCaption;
   pPS->pDlgParent = pDlgParent;

   m_pages.Add(pPS);

   return true;
}

/////////////////////////////////////////////////////////////////////////////

BOOL CPrefsDialog::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
      return FALSE;

   cs.lpszClass = AfxRegisterWndClass(CS_DBLCLKS, NULL, NULL, NULL);
   cs.style |= WS_CLIPCHILDREN;
   return TRUE;
}

/////////////////////////////////////////////////////////////////////////////

bool CPrefsDialog::ShowPage(int iPage)
{
   m_captionBar.SetWindowText("");

	// turn off the current page
   if ((m_iCurPage >= 0) && (m_iCurPage < m_pages.GetSize()))
   {
      pageStruct *pPS = (pageStruct *)m_pages.GetAt(m_iCurPage);
      ASSERT(pPS);
      if (pPS)
      {
         pPS->pDlg->ShowWindow(SW_HIDE);
      }
      else
      {
         return false;
      }
   }

	// show the new one
   if ((iPage >= 0) && (iPage < m_pages.GetSize()))
   {
      pageStruct *pPS = (pageStruct *)m_pages.GetAt(iPage);
      ASSERT(pPS);
      if (pPS)
      {
			// update caption bar
         m_captionBar.SetWindowText(pPS->csCaption);

         pPS->pDlg->MoveWindow(m_frameRect.left, m_frameRect.top, m_frameRect.Width(), m_frameRect.Height());
         pPS->pDlg->ShowWindow(SW_SHOW);
         pPS->pDlg->SetFocus();

         return true;
      }
   }

   return false;
}

/////////////////////////////////////////////////////////////////////////////

void CPrefsDialog::OnOK() 
{
	// tell all of the sub-dialogs "OK"
   for (int i=0;i<m_pages.GetSize();i++)
   {
      pageStruct *pPS = (pageStruct *)m_pages.GetAt(i);
      if (pPS)
      {
         pPS->pDlg->UpdateData(TRUE);
         pPS->pDlg->EndDialog(IDOK);
      }
   }

	CDialog::OnOK();
}

/////////////////////////////////////////////////////////////////////////////

void CPrefsDialog::OnCancel() 
{
	// tell all of the sub-dialogs "Cancel"
   for (int i=0;i<m_pages.GetSize();i++)
   {
      pageStruct *pPS = (pageStruct *)m_pages.GetAt(i);
      if (pPS)
      {
         pPS->pDlg->EndDialog(IDCANCEL);
      }
   }

	CDialog::OnCancel();
}

/////////////////////////////////////////////////////////////////////////////

void CPrefsDialog::OnSelchangedPageTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

   if (pNMTreeView->itemNew.lParam)
   {
		// find out which page was selected
      int iIdx = -1;
      for (int i=0;i<m_pages.GetSize();i++)
      {
         if (m_pages.GetAt(i)==(pageStruct *)pNMTreeView->itemNew.lParam)
         {
            iIdx = i;
            break;
         }
      }

		// show that page
      if ((iIdx >= 0) && (iIdx < m_pages.GetSize()))
      {
         pageStruct *pPS = (pageStruct *)m_pages.GetAt(iIdx);
         if (m_iCurPage!=iIdx)
         {
				PostMessage(WM_CHANGE_PAGE, iIdx);
         }
      }
   }

	*pResult = 0;
}

/////////////////////////////////////////////////////////////////////////////

void CPrefsDialog::OnGetdispinfoPageTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	TV_DISPINFO* pTVDispInfo = (TV_DISPINFO*)pNMHDR;

	// return the caption of the appropriate dialog
   if (pTVDispInfo->item.lParam)
   {
      if (pTVDispInfo->item.mask & TVIF_TEXT)
      {
         pageStruct *pPS = (pageStruct *)pTVDispInfo->item.lParam;
         strcpy(pTVDispInfo->item.pszText, pPS->csCaption);
      }
   }
   
	*pResult = 0;
}

/////////////////////////////////////////////////////////////////////////////

void CPrefsDialog::OnPhelp() 
{
	// simulate the property sheet method of sending Help (with WM_NOTIFY)
   if ((m_iCurPage >= 0) && (m_iCurPage < m_pages.GetSize()))
   {
      pageStruct *pPS = (pageStruct *)m_pages.GetAt(m_iCurPage);
      ASSERT(pPS);
      if (pPS)
      {
			NMHDR nm;
			nm.code=PSN_HELP;
			nm.hwndFrom=m_hWnd;
			nm.idFrom=CPrefsDialog::IDD;
			pPS->pDlg->SendMessage(WM_NOTIFY, 0, (long)&nm);
		}
	}
}
